# -*- coding: utf-8 -*-

# Copyright (c) 2006 Detlev Offenbach <detlev@die-offenbachs.de>
#

"""
Module implementing a canvas item subclass for an arrow.
"""

from math import *

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from E4CanvasLine import E4CanvasLine

NormalArrow = 1
WideArrow = 2

class E4CanvasArrow(E4CanvasLine):
    """
    Class implementing an arrow canvas item subclass.
    """
    def __init__(self, canvas, origin = QPointF(0.0, 0.0), end = QPointF(0.0, 0.0),
                 filled = False, type = NormalArrow):
        """
        Constructor
        
        @param canvas reference to the canvas object (E4Canvas)
        @param origin origin of the arrow (QPointF)
        @param end end point of the arrow (QPointF)
        @param filled flag indicating a filled arrow head (boolean)
        @param type arrow type (NormalArrow, WideArrow)
        """
        E4CanvasLine.__init__(self, canvas, origin, end)
        
        self._filled = filled
        self._type = type
        
        self._halfLength = 13
        
    def drawShape(self, painter):
        """
        Public method to draw the shape.
        
        @param painter reference to the painter object (QPainter)
        """
        if not self.isVisible():
            return
        
        painter.setPen(self.getPen())
        if self._filled:
            painter.setBrush(self.getPen().brush())
        
        if self._path.isEmpty():
            arrowAngle = self._type * 0.5 * atan(sqrt(3.0) / 3.0);
            slope = atan2(self._end.y() - self.y(), 
                          self._end.x() - self.x()) # slope of line
            
            # Calculate left arrow point
            arrowSlope = slope + arrowAngle;
            a1 = QPointF(self._end.x() - self._halfLength * cos(arrowSlope),
                         self._end.y() - self._halfLength * sin(arrowSlope))
            
            # Calculate right arrow point
            arrowSlope = slope - arrowAngle;
            a2 = QPointF(self._end.x() - self._halfLength * cos(arrowSlope),
                         self._end.y() - self._halfLength * sin(arrowSlope))
            
            # Calculate intersection point
            isp = QPointF((a1.x() + a2.x()) / 2.0, (a1.y() + a2.y()) / 2.0)
            
            # now do the drawing
            self._path.moveTo(self.x(), self.y())
            self._path.lineTo(isp.x(), isp.y())
            self._path.lineTo(a1.x(), a1.y())
            self._path.lineTo(self._end.x(), self._end.y())
            self._path.lineTo(a2.x(), a2.y())
            self._path.lineTo(isp.x(), isp.y())
            self._boundingRect = self._path.boundingRect()
        
        painter.drawPath(self._path)
        if self.isSelected():
            self.drawSelected(painter)
