# -*- coding: utf-8 -*-

# Copyright (c) 2006 Detlev Offenbach <detlev@die-offenbachs.de>
#

"""
Module implementing a canvas item subclass for a rectangle.
"""

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from E4CanvasItem import E4CanvasItem

class E4CanvasRectangle(E4CanvasItem):
    """
    Class implementing a rectangle canvas item subclass.
    """
    def __init__(self, canvas, origin = QPointF(0.0, 0.0), size = QSizeF(0.0, 0.0)):
        """
        Constructor
        
        @param canvas reference to the canvas object (E4Canvas)
        @param origin origin of the rectangle (QPointF)
        @param size size of the rectangle (QSizeF)
        """
        E4CanvasItem.__init__(self, canvas, origin)
        
        self._size = QSizeF(size.width(), size.height())
        
    def setSize(self, width, height):
        """
        Public method to set the rectangles size.
        
        @param width width of the rectangle (float)
        @param height height of the rectangle (float)
        """
        self._size.setWidth(width)
        self._size.setHeight(height)
        self._path = QPainterPath()
        
    def width(self):
        """
        Public method to get the width of the rectangle.
        
        @return width of the rectangle (float)
        """
        return self._size.width()
        
    def height(self):
        """
        Public method to get the height of the rectangle.
        
        @return height of the rectangle (float)
        """
        return self._size.height()
        
    def drawShape(self, painter):
        """
        Public method to draw the shape.
        
        @param painter reference to the painter object (QPainter)
        """
        if not self.isVisible():
            return
        
        painter.setPen(self.getPen())
        
        if self._path.isEmpty():
            self._path.addRect(self.x(), self.y(),
                               self._size.width(), self._size.height())
            self._boundingRect = self._path.boundingRect()
        
        painter.drawPath(self._path)
        if self.isSelected():
            self.drawSelected(painter)
