# -*- coding: utf-8 -*-

# Copyright (c) 2006 Detlev Offenbach <detlev@die-offenbachs.de>
#

"""
Module implementing the Programs page.
"""

import os
import sys
import subprocess

from PyQt4.QtCore import pyqtSignature, QString, QStringList, Qt
from PyQt4.QtGui import QApplication, QTreeWidgetItem, QHeaderView, QCursor

from ConfigurationPageBase import ConfigurationPageBase
from Ui_ProgramsPage import Ui_ProgramsPage

import Preferences
import Utilities

class ProgramsPage(ConfigurationPageBase, Ui_ProgramsPage):
    """
    Class implementing the Programs page.
    """
    def __init__(self, dlg):
        """
        Constructor
        
        @param dlg reference to the configuration dialog
        """
        ConfigurationPageBase.__init__(self)
        self.setupUi(self)
        self.setObjectName("ProgramsPage")
        
        self.configDlg = dlg
        
    def save(self):
        """
        Public slot to save the configuration (do nothing implementation).
        """
        return
        
    @pyqtSignature("")
    def on_programsSearchButton_clicked(self):
        """
        Private slot to search for all supported/required programs.
        """
        QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
        QApplication.processEvents()
        
        self.programsList.clear()
        header = self.programsList.header()
        header.setSortIndicator(0, Qt.AscendingOrder)
        header.setSortIndicatorShown(False)
        
        # 1. do the Qt programs
        qtPage = self.configDlg.getPage("qtPage")
        if qtPage:
            qtdir = qtPage.qtEdit.text()
        else:
            qtdir = QString()
        if qtdir.isEmpty():
            qtdir = Preferences.getQtDir()
        else:
            qtdir = unicode(qtdir)
        # 1a. Translation Converter
        exe = sys.platform == "win32" and "lrelease.exe" or "lrelease"
        if qtdir:
            exe = os.path.join(qtdir, 'bin', exe)
        version = self.__createProgramEntry(self.trUtf8("Translation Converter (Qt3)"), 
                    exe, '-version', 'lrelease', -1)
        # 1b. Qt Designer
        exe = sys.platform == "win32" and "designer.exe" or "designer"
        if qtdir:
            exe = os.path.join(qtdir, 'bin', exe)
        self.__createProgramEntry(self.trUtf8("Qt3 Designer"), exe, version = version)
        # 1c. Qt Linguist
        exe = sys.platform == "win32" and "linguist.exe" or "linguist"
        if qtdir:
            exe = os.path.join(qtdir, 'bin', exe)
        self.__createProgramEntry(self.trUtf8("Qt3 Linguist"), exe, version = version)
        # 1d. Qt Assistant
        exe = sys.platform == "win32" and "assistant.exe" or "assistant"
        if qtdir:
            exe = os.path.join(qtdir, 'bin', exe)
        self.__createProgramEntry(self.trUtf8("Qt3 Assistant"), exe, version = version)
        
        # do the Qt4 programs
        if qtPage:
            qt4dir = qtPage.qt4Edit.text()
        else:
            qt4dir = QString()
        if qt4dir.isEmpty():
            qt4dir = Preferences.getQt4Dir()
        else:
            qt4dir = unicode(qt4dir)
        # 1e. Translation Converter
        exe = sys.platform == "win32" and "lrelease.exe" or "lrelease"
        if qt4dir:
            exe = os.path.join(qt4dir, 'bin', exe)
        version = self.__createProgramEntry(self.trUtf8("Translation Converter (Qt4)"), 
                    exe, '-version', 'lrelease', -1)
        # 1f. Qt4 Designer
        exe = sys.platform == "win32" and "designer.exe" or "designer"
        if qt4dir:
            exe = os.path.join(qt4dir, 'bin', exe)
        self.__createProgramEntry(self.trUtf8("Qt4 Designer"), exe, version = version)
        # 1g. Qt4 Linguist
        exe = sys.platform == "win32" and "linguist.exe" or "linguist"
        if qt4dir:
            exe = os.path.join(qt4dir, 'bin', exe)
        self.__createProgramEntry(self.trUtf8("Qt4 Linguist"), exe, version = version)
        # 1h. Qt4 Assistant
        exe = sys.platform == "win32" and "assistant.exe" or "assistant"
        if qt4dir:
            exe = os.path.join(qt4dir, 'bin', exe)
        self.__createProgramEntry(self.trUtf8("Qt4 Assistant"), exe, version = version)
        
        # 2. do the PyQt programs
        # 2a. Translation Extractor
        self.__createProgramEntry(self.trUtf8("Translation Extractor (Python, Qt3)"), 
            sys.platform == "win32" and "pylupdate.exe" or "pylupdate", 
            '-version', 'pylupdate', -1)
        # 2b. Forms Compiler
        self.__createProgramEntry(self.trUtf8("Forms Compiler (Python, Qt3)"), 
            sys.platform == "win32" and "pyuic.exe" or "pyuic", 
            '-version', 'Python User', 4)
        # 2c. Translation Extractor PyQt4
        self.__createProgramEntry(self.trUtf8("Translation Extractor (Python, Qt4)"), 
            sys.platform == "win32" and "pylupdate4.exe" or "pylupdate4", 
            '-version', 'pylupdate', -1)
        # 2d. Forms Compiler PyQt4
        self.__createProgramEntry(self.trUtf8("Forms Compiler (Python, Qt4)"), 
            sys.platform == "win32" and "pyuic4.bat" or "pyuic4", 
            '--version', 'Python User', 4)
        # 2e. Resource Compiler PyQt4
        self.__createProgramEntry(self.trUtf8("Resource Compiler (Python, Qt4)"), 
            sys.platform == "win32" and "pyrcc4.exe" or "pyrcc4", 
            '-version', 'Resource Compiler', -1)
        
        # 3. do the Ruby programs
        # 3a. Forms Compiler
        self.__createProgramEntry(self.trUtf8("Forms Compiler (Ruby, Qt3)"), 
            sys.platform == "win32" and "rbuic.exe" or "rbuic", 
            '-version', 'QtRuby User', 4)
        # 3b. Forms Compiler for Qt4
        self.__createProgramEntry(self.trUtf8("Forms Compiler (Ruby, Qt4)"), 
            sys.platform == "win32" and "rbuic4.exe" or "rbuic4", 
            '-version', 'Qt', -1)
        # 3b. Resource Compiler for Qt4
        self.__createProgramEntry(self.trUtf8("Resource Compiler (Ruby, Qt4)"), 
            sys.platform == "win32" and "rbrcc.exe" or "rbrcc", 
            '-version', 'Ruby Resource Compiler', -1)
        
        # 4. do the Eric4 programs
        # 4a. API File Generator
        self.__createProgramEntry(self.trUtf8("Eric4 API File Generator"), 
            sys.platform == "win32" and "eric4-api.bat" or "eric4-api", 
            '--version', 'eric4-', -1)
        # 4b. Documentation Generator
        self.__createProgramEntry(self.trUtf8("Eric4 Documentation Generator"), 
            sys.platform == "win32" and "eric4-doc.bat" or "eric4-doc", 
            '--version', 'eric4-', -1)
        # 4c. Translation Previewer
        self.__createProgramEntry(self.trUtf8("Eric4 Translation Previewer"), 
            sys.platform == "win32" and "eric4-trpreviewer.bat" or "eric4-trpreviewer", 
            '--version', 'Eric4', -1)
        # 4d. Forms Previewer
        self.__createProgramEntry(self.trUtf8("Eric4 Forms Previewer"), 
            sys.platform == "win32" and "eric4-uipreviewer.bat" or "eric4-uipreviewer", 
            '--version', 'Eric4', -1)
        
        # 5. do the CORBA programs
        # 5a. omniORB
        corbaPage = self.configDlg.getPage("corbaPage")
        if corbaPage:
            exe = unicode(corbaPage.idlEdit.text())
        else:
            exe = ""
        if not exe:
            exe = unicode(Preferences.getCorba("omniidl"))
            if sys.platform == "win32":
                exe += ".exe"
        self.__createProgramEntry(self.trUtf8("CORBA IDL Compiler"), exe,
            '-V', 'omniidl', -1)
        
        # 6. do the Version Control programs
        # 6a. CVS
        self.__createProgramEntry(self.trUtf8("Version Control - CVS"), 
            sys.platform == "win32" and "cvs.exe" or "cvs", 
            '--version', 'Concurrent Versions', 4)
        # 6b. Subversion
        self.__createProgramEntry(self.trUtf8("Version Control - Subversion"), 
            sys.platform == "win32" and "svn.exe" or "svn", 
            '--version', 'svn', 2)
        # 6c. Subversion - PySvn
        try:
            import pysvn
            try:
                entryText = os.path.dirname(pysvn.__file__)
            except AttributeError:
                entryText = "PySvn"
            self.__createEntry(self.trUtf8("Version Control - Subversion (Python)"),
                entryText, ".".join([str(v) for v in pysvn.version]))
        except ImportError:
            self.__createEntry(self.trUtf8("Version Control - Subversion (Python)"),
                "PySvn", "")
        
        # 7. do the Checker programs
        # 7a. Pylint
        self.__createProgramEntry(self.trUtf8("Checkers - Pylint"), 
            sys.platform == "win32" and "pylint.bat" or "pylint", 
            '--version', 'pylint', -1)
        
        # 8. do the Packagers programs
        # 8a. cx_freeze
        self.__createProgramEntry(self.trUtf8("Packagers - cx_freeze"), 
            sys.platform == "win32" and "FreezePython.exe" or "FreezePython", 
            '--version', 'FreezePython', -1)
        
        self.programsList.sortByColumn(0, Qt.AscendingOrder)
        QApplication.restoreOverrideCursor()

    def __createProgramEntry(self, description, exe,
                             versionCommand = "", versionStartsWith = "", 
                             versionPosition = 0, version = ""):
        """
        Private method to generate a program entry.
        
        @param description descriptive text (string or QString)
        @param exe name of the executable program (string)
        @param versionCommand command line switch to get the version info (string).
            if this is empty, the given version will be shown.
        @param versionStartsWith start of line identifying version info (string)
        @param versionPosition index of part containing the version info (integer)
        @keyparam version version string to show (string).
        @return version string of detected or given version (string)
        """
        itm = QTreeWidgetItem(self.programsList, QStringList(description))
        font = itm.font(0)
        font.setBold(True)
        itm.setFont(0, font)
        if not exe:
            itm.setText(1, self.trUtf8("(not configured)"))
        else:
            if os.path.isabs(exe):
                if not Utilities.isExecutable(exe):
                    exe = ""
            else:
                exe = Utilities.getExecutablePath(exe)
            if exe:
                if versionCommand and versionStartsWith and versionPosition:
                    try:
                        p = subprocess.Popen("%s %s" % (exe, versionCommand),
                            shell = True, 
                            stdin = subprocess.PIPE, 
                            stdout = subprocess.PIPE, 
                            stderr = subprocess.STDOUT, 
                            close_fds = True).stdout
                        lines = p.readlines()
                        p.close()
                        for line in lines:
                            if line.startswith(versionStartsWith):
                                version = line.split()[versionPosition]
                                break
                    except OSError:
                        version = self.trUtf8("(not executable)")
                itm2 = QTreeWidgetItem(itm, QStringList() << exe << version)
                self.programsList.setItemExpanded(itm, True)
            else:
                itm.setText(1, self.trUtf8("(not found)"))
        QApplication.processEvents()
        self.programsList.header().resizeSections(QHeaderView.ResizeToContents)
        self.programsList.header().setStretchLastSection(True)
        return version
        
    def __createEntry(self, description, entryText, entryVersion):
        """
        Private method to generate a program entry.
        
        @param description descriptive text (string or QString)
        @param entryText text to show (string or QString)
        @param entryVersion version string to show (string or QString).
        """
        itm = QTreeWidgetItem(self.programsList, QStringList(description))
        font = itm.font(0)
        font.setBold(True)
        itm.setFont(0, font)
        
        if entryVersion:
            itm2 = QTreeWidgetItem(itm, QStringList() << entryText << entryVersion)
            self.programsList.setItemExpanded(itm, True)
        else:
            itm.setText(1, self.trUtf8("(not found)"))
        QApplication.processEvents()
        self.programsList.header().resizeSections(QHeaderView.ResizeToContents)
        self.programsList.header().setStretchLastSection(True)

def create(dlg):
    """
    Module function to create the configuration page.
    
    @param dlg reference to the configuration dialog
    """
    page = ProgramsPage(dlg)
    return page
