# -*- coding: utf-8 -*-

# Copyright (c) 2002 - 2006 Detlev Offenbach <detlev@die-offenbachs.de>
#

"""
Module implementing a dialog to show the found files to the user.
"""

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from Ui_AddFoundFilesDialog import Ui_AddFoundFilesDialog

class AddFoundFilesDialog(QDialog, Ui_AddFoundFilesDialog):
    """
    Class implementing a dialog to show the found files to the user.
    
    The found files are displayed in a listview.
    Pressing the 'Add All' button adds all files to the current project,
    the 'Add Selected' button adds only the selected files and the 'Cancel'
    button cancels the operation.
    """
    
    def __init__(self, files, parent = None, name = None):
        """
        Constructor
        
        @param files list of files, that have been found for addition (QStringList)
        @param parent parent widget of this dialog (QWidget)
        @param name name of this dialog (string or QString)
        """
        QDialog.__init__(self, parent)
        if name:
            self.setObjectName(name)
        self.setupUi(self)
        
        self.fileList.addItems(files)
        
    @pyqtSignature("")
    def on_addAllButton_clicked(self):
        """
        Private slot to handle the 'Add All' button press. 
        
        @return always 1 (int)
        """
        self.done(1)
        
    @pyqtSignature("")
    def on_addSelectedButton_clicked(self):
        """
        Private slot to handle the 'Add Selected' button press.
        
        @return always 2 (int)
        """
        self.done(2)
        
    def getSelection(self):
        """
        Public method to return the selected items.
        
        @return list of selected files (QStringList)
        """
        list = QStringList()
        for itm in self.fileList.selectedItems():
            list.append(itm.text())
        return list
